/*-
 * #%L
 * Elastic APM Java agent
 * %%
 * Copyright (C) 2018 - 2020 Elastic and contributors
 * %%
 * Licensed to Elasticsearch B.V. under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. Elasticsearch B.V. licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * #L%
 */
package co.elastic.apm.agent.process;

import co.elastic.apm.agent.impl.transaction.TraceContextHolder;
import net.bytebuddy.asm.Advice;
import net.bytebuddy.description.method.MethodDescription;
import net.bytebuddy.description.type.TypeDescription;
import net.bytebuddy.matcher.ElementMatcher;

import javax.annotation.Nullable;

import static net.bytebuddy.matcher.ElementMatchers.named;
import static net.bytebuddy.matcher.ElementMatchers.takesArguments;

/**
 * Instruments {@link ProcessBuilder#start()}
 */
public class ProcessStartInstrumentation extends BaseProcessInstrumentation {

    @Override
    public ElementMatcher<? super TypeDescription> getTypeMatcher() {
        return named("java.lang.ProcessBuilder");
    }

    @Override
    public ElementMatcher<? super MethodDescription> getMethodMatcher() {
        return named("start")
            .and(takesArguments(0));
    }

    @Override
    public Class<?> getAdviceClass() {
        return ProcessBuilderStartAdvice.class;
    }

    public static class ProcessBuilderStartAdvice {

        @Advice.OnMethodExit(suppress = Throwable.class, onThrowable = Throwable.class)
        public static void onExit(@Advice.This ProcessBuilder processBuilder,
                                  @Advice.Return Process process,
                                  @Advice.Thrown @Nullable Throwable t) {

            if (tracer == null) {
                return;
            }
            TraceContextHolder<?> parentSpan = tracer.getActive();
            if (parentSpan == null) {
                return;
            }

            if (t != null) {
                // unable to start process, report exception as it's likely to be a bug
                parentSpan.captureException(t);
            }

            ProcessHelper.startProcess(parentSpan, process, processBuilder.command());
        }
    }
}
